<!---
I provide two fuzzy date functions, one for formatting timestamps
themselves, and one for formatting timespans.

------------------------------------------------------------------------
$Id: udflib_fuzzydate.cfm 2321 2007-06-02 06:16:40Z barneyb $

Copyright 2007 Barney Boisvert (bboisvert@gmail.com).

Licensed under the Apache License, Version 2.0 (the "License"); you may
not use this file except in compliance with the License. You may obtain
a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
--->
<cfset NOW = now() />
<cfset FUZZY_TODAY = createDate(year(NOW), month(NOW), day(NOW)) />
<cfset FUZZY_TIME_CUTOFF = dateAdd("h", -22, NOW) />
<cfset FUZZY_YESTERDAY = dateAdd("d", -1, FUZZY_TODAY) />
<cfset FUZZY_DELTA_CUTOFF = dateAdd("d", -7, FUZZY_TODAY) />
<cfset FUZZY_NEW_YEARS_DAY = createDate(year(FUZZY_TODAY), 1, 1) />
<cffunction name="fuzzyDateFormat" output="false" returntype="string"
	hint="I accept a datetime/timestamp and format it in a fuzzy way.  For example, very recent timestamps just display times, older ones display month/day strings, and really old ones display month/day/year strings.">
	<cfargument name="date" type="date" required="true" />
	<cfset var result = "" />
	<cfif date GTE FUZZY_TIME_CUTOFF>
		<cfset result = lCase(timeFormat(date, "h:mm tt")) />
	<cfelseif date GTE FUZZY_YESTERDAY>
		<cfset result = "ystrdy" />
	<cfelseif date GTE FUZZY_NEW_YEARS_DAY>
		<cfset result = dateFormat(date, "mmm d") />
		<cfif date GTE FUZZY_DELTA_CUTOFF>
			<cfset result = result & " [-#1 + dateDiff('d', date, FUZZY_TODAY)#]" />
		</cfif>
	<cfelse>
		<cfset result = dateFormat(date, "m/d/yy") />
	</cfif>
	<cfreturn result />
</cffunction>



<cffunction name="fuzzyTimespanFormat" output="false" returntype="string"
	hint="I format a timespan (passed as seconds) into the largest unit that can be used to express it in whole numbers.">
	<cfargument name="delta" type="numeric" required="true"
		hint="the timespan, in seconds" />
	<cfif delta LTE 60>
		<cfreturn assembleFuzzyTimespanFormat(delta, "sec", "") />
	</cfif>
	<cfset delta = delta / 60 /> <!--- to minutes --->
	<cfif delta LTE 60>
		<cfreturn assembleFuzzyTimespanFormat(round(delta), "min", "") />
	</cfif>
	<cfset delta = delta / 60 /> <!--- to hours --->
	<cfif delta LTE 24>
		<cfreturn assembleFuzzyTimespanFormat(round(delta), "hr") />
	</cfif>
	<cfset delta = delta / 24 /> <!--- to days --->
	<cfif delta LTE 7>
		<cfreturn assembleFuzzyTimespanFormat(round(delta), "day") />
	</cfif>
	<cfif delta LTE 30>
		<cfreturn assembleFuzzyTimespanFormat(round(delta / 7), "wk") />
	</cfif>
	<cfset delta = delta / 30 /> <!--- to months --->
	<cfif delta LTE 12>
		<cfreturn assembleFuzzyTimespanFormat(round(delta), "mo") />
	</cfif>
	<cfreturn assembleFuzzyTimespanFormat(round(delta / 12), "yr") />
</cffunction>
<cffunction name="assembleFuzzyTimespanFormat" output="false" returntype="string"
	hint="I am a private helper function for fuzzyTimespanFormat.  I could be inlined, but code dupication is evil.">
	<cfargument name="count" type="numeric" required="true" />
	<cfargument name="unit" type="string" required="true" />
	<cfargument name="plural" type="string" default="s" />
	<cfset var result = count & " " & unit />
	<cfif count NEQ 1>
		<cfset result = result & plural />
	</cfif>
	<cfreturn result />
</cffunction>